# coding: utf-8
u"""Инструментарий разработчика веб-приложений на платформе M3."""
import abc


class Config(object):

    u"""Конфигурация приложения."""

    __metaclass__ = abc.ABCMeta

    @abc.abstractproperty
    def controller(self):
        u"""Контроллер, в котором будут регистрироваться паки приложения.

        :rtype: :class:`m3.actions.ActionController`
        """

    @abc.abstractproperty
    def observer(self):
        u"""Наблюдатель objectpack.

        :rtype: :class:`objectpack.observer.base.Observer`
        """


#: Конфигурация приложения ``m3_dev_utils``.
#:
#: В проекте, который использует данное приложение, в этой переменной должен
#: быть сохранен экземпляр потомка класса :class:`~m3_dev_utils.Config`.
#:
#: Пример создания класса с конфигурацией и настройки приложения:
#:
#: .. code-block:: python
#:    :caption: ``extedu/apps.py``
#:
#:    class AppConfig(AppConfig):
#:
#:        def _init_m3_dev_utils(self):
#:            if 'm3_dev_utils' in settings.INSTALLED_APPS:
#:                import m3_dev_utils
#:
#:                class Config(m3_dev_utils.Config):
#:
#:                    @cached_property
#:                    def controller(self):
#:                        from extedu.controllers import main_controller
#:                        return main_controller
#:
#:                    @cached_property
#:                    def observer(self):
#:                        from extedu.controllers import core_observer
#:                        return core_observer
#:
#:                m3_dev_utils.config = Config()
#:
#:        def ready(self):
#:            self._init_m3_dev_utils()
#:
#:            super(AppConfig, self).ready()
config = None


def get_config():
    u"""Возвращает конфигурацию приложения.

    :rtype: m3_dev_utils.Config
    """
    # pylint: disable=global-statement
    global config
    assert config
    return config
